//Boston University - Electronics Design Facility
//Monkey Timer Mk2
//http://edf.bu.edu/
//written by Chris Lawlor, January - April 2011
//Device is Atmega2560
//CPU clock is 16Mhz
//This file defines the serial communication functions


#ifndef _ATMEGA2560_SERIAL_COM_H
#define _ATMEGA2560_SERIAL_COM_H 1 

#include <ctype.h>
#include <string.h>

#define CR 0x0D //ascii carrage return
#define LF 0x0A //ascii linefeed
#define BS 0x08 //ascii backspace

#define USART_BUFFMAX 32
char usart_buffer[USART_BUFFMAX];


void ClearBuffer(char *c, int size)
{
	memset(c, 0, size);
}


void USART_setup()
//57600 baud
//8 data bits
//1 stop bit
//0 parity bits
{
	UBRR0  = 0x10;
	UCSR0B = 0x18;
	UCSR0C = 0x06;
}

char USART_ReceiveChar(result_t (*breakout_handler)(), result_t *breakout)
//the breakout_handler should be a pointer to a non-blocking function that checks an 
//internal state like carrier disconnection or button press, NULL is also acceptable
//*breakout will contain the return value of the breakout_handler, or NULL is accaptable
//if either breakout_handler or breakout is NULL, breakout handling will be skipped.
{	

	if (breakout!=NULL)
		*breakout = 0;

	while(!(UCSR0A & (1<<RXC0)) && !((breakout==NULL)?(0):(*breakout)) )
	{
		if ((breakout_handler!=NULL)&&(breakout!=NULL))
			*breakout = breakout_handler();
	}
	
	return toupper(UDR0);
}

void USART_SendChar(char c)
{
	while(!(UCSR0A & (1<<UDRE0)))
	{}
	UDR0 = c;
}

void USART_SendLineFeed()
{
	USART_SendChar(CR);
	USART_SendChar(LF);
}

void USART_SendString(char *s, int N)
//if N>0, then N characters will be send
//otherwise, if N<=0 transmission will continue 
//until a null characher is reached
{
	int i = N - 1;
	
	while ( ((N<=0)&&(*s != '\0')) || ((N>0)&&(i>=0)) )
	{
		if (*s == '\n')
			USART_SendLineFeed();
		else
			USART_SendChar(*s);
		s++;
		i--;
	}
}

int USART_ReceiveString(char *s, int smax, result_t (*breakout_handler)(), result_t *breakout)
//returns the length of the received string on success 
//returns <0 on error, eg. -1 for buffer overflow
//the breakout_handler should be a pointer to a non-blocking function that checks an 
//internal state like carrier disconnection or button press, NULL is also acceptable
//*breakout will contain the return value of the breakout_handler, or NULL is accaptable
//if either breakout_handler or breakout is passed in NULL, breakout handling will be skipped.
{
	int lf_count = 0;
	int cr_count = 0;
	int8_t i = 0;
	char temp;	
	
	if (breakout!=NULL)
		*breakout = 0;
	
	for (i = 0; (i<smax) && (lf_count<1) && (cr_count<1) ; i++ )
	{
		temp = USART_ReceiveChar(breakout_handler, breakout);
		if (temp == LF)
		{
			lf_count++;
		}
		else if (temp == CR)
		{
			cr_count++;
		}
		
		if (temp != LF && temp != CR)
			s[i] = temp;

		if ((breakout!=NULL)&&(*breakout==1))
		{
			i++;
			break;
		}
		if (temp==BS)
		{
			if (i==0)
			{
				i-=1;
			}
			else
			{
				i-=2;		
			}
		}
	}

	if (i < smax)
	{   
		s[i-1] = '\0';
		return i-1;  //ie, the strlen of s
	}
	else
	{
		s[smax-1] = '\0';	//truncate the string at the end
		return -1;
	}
}

void USART_echotest()
{

#define EXIT_CODE "EXIT"
	//char cb[USART_BUFFMAX];
	USART_SendString("TYPE '", 0);
	USART_SendString(EXIT_CODE, 0);
	USART_SendString("' TO QUIT ECHO MODE", 0);
	USART_SendLineFeed();
	USART_SendString("ECHO ON", 0);
	USART_SendLineFeed();
	ClearBuffer(usart_buffer, USART_BUFFMAX);
	while( strncmp((usart_buffer), EXIT_CODE, strlen(EXIT_CODE)) != 0)
	{
		ClearBuffer(usart_buffer, USART_BUFFMAX);
		USART_ReceiveString(usart_buffer, USART_BUFFMAX, NULL, NULL);
		USART_SendString(usart_buffer, 0);
		USART_SendLineFeed();
		//snprintf(cb, USART_BUFFMAX, "%02x, %02x, %02x, %02x, %02x, %02x, %02x, %02x, %02x, %02x ", usart_buffer[0], usart_buffer[1], usart_buffer[2], usart_buffer[3], usart_buffer[4], usart_buffer[5], usart_buffer[6], usart_buffer[7], usart_buffer[8], usart_buffer[9] );
		//USART_SendString(cb, 0);
	}
	USART_SendString("ECHO OFF", 0);
	USART_SendLineFeed();
}

/*


#define BU 			"Boston University"
#define EDF			"Electronics Design Facility"
#define PRODUCT		"your product name"
#define	FWV			"firmware v0.0"



void SendHelp()
//	Commands
//  ---------------
//	1. identify 
//	2. help
//	3. echo
//	4. start <L|R>
//	5. cancel
//	6. download
//	7. init
{
	USART_SendString("Valid commands are: ID, HELP, ECHO, START <L|R>, CANCEL, DOWNLOAD, INIT, SET", 0);
}


void NotYetImplemented()
{
	USART_SendString("NOT YET IMPLEMENTED", 0);
}

void SendIdentity()
{
	USART_SendString(BU, 0);
	USART_SendLineFeed();

	USART_SendString(EDF, 0);
	USART_SendLineFeed();

	USART_SendString(PRODUCT, 0);
	USART_SendLineFeed();

	USART_SendString(FWV, 0);
	USART_SendLineFeed();

}



#define PROMPT 		"?>"

void terminal_mode()
{
	uint32_t tc1 = 0;
	uint32_t tc2 = 0;

	while(1)
	{
		USART_SendLineFeed();
		USART_SendString(PROMPT, 0);
		ClearBuffer(usart_buffer, USART_BUFFMAX);
		USART_ReceiveString(usart_buffer, USART_BUFFMAX);	
		USART_SendLineFeed(0);
		
		if (strlen(usart_buffer) == 0)
		{
			//do nothing
		}
		else if (strncmp(usart_buffer, "ID", 2) == 0)
		{
			SendIdentity(0);	
		}
		else if (strncmp(usart_buffer, "HELP", 4) == 0)
		{
			SendHelp();
		}
		else if (strncmp(usart_buffer, "ECHO", 4) == 0)
		{

			USART_echotest();
		}
		else if (strncmp(usart_buffer, "START", 5) == 0)
		{
			NotYetImplemented();
		}
		else if (strncmp(usart_buffer, "CANCEL", 6) == 0)
		{
			NotYetImplemented();
		}
		else if (strncmp(usart_buffer, "DOWNLOAD", 8) == 0)
		{
			NotYetImplemented();
		}
		
		else if (strncmp(usart_buffer, "SET", 4) == 0)
		{
			NotYetImplemented();
		}

		
		else if (strncmp(usart_buffer, "INIT", 4) == 0)
		{
			NotYetImplemented();
		}

		else 
		{
			USART_SendString("UNKNOWN COMMAND.  TRY 'HELP'", 0);
		}
	
	}//while(1)

}
*/

#endif


