//Boston University - Electronics Design Facility
//Monkey Timer Mk2
//http://edf.bu.edu/
//written by Chris Lawlor, January - April 2011
//Device is Atmega2560
//CPU clock is 16Mhz
//This file contains the Menu Definitions for the Menuing Framework


#ifndef _MONKEY_MK2_MENU_FRAMEWORK_H
#define _MONKEY_MK2_MENU_FRAMEWORK_H



// ADDING AND REMOVING MENUS
//
// To add a new menu at number k, do 4 things: 
//
// 1. add its menu definition in this MENU DEFINITION FILE (ie #define MENU_k_NUM, etc.)
// 2. add a call to populate_menu(MENU(k)) in the render_menus() function in the MENU FRAMEWORK FILE
// 3. increase the NUM_MENUS (#define constant) accordingly if necessary (and watch out for the RAM usage!)
// 4. now the menu can be accessed using push_menu(k) 
//
// To remove a menu, undo these 4 steps



//MENU_n_FLAGS bit definitions are as follows:
//   
//
//	| 7 | 6 |    5    |    4    |    3    |    2    |   1   |   0   |
//	| - | - | SELENL3 | SELENL2 | SELENL1 | SELENL0 | DFLT1 | DFLT0 |
//
//
//	FLAGS0:	DFLT0  		lower  bit of the default line, (the values range from 0 to 3)
//	FLAGS1:	DFLT1  		higher bit of the default line
//	FLAGS2:	SELENL0  	(see note 1) set to 1 it enables MENU line 0 to be selected in the menu, 0 disables selection of the line
//	FLAGS3:	SELENL1  	(see note 1) set to 1 it enables MENU line 1 to be selected in the menu, 0 disables selection of the line
//	FLAGS4:	SELENL2  	(see note 1) set to 1 it enables MENU line 2 to be selected in the menu, 0 disables selection of the line
//	FLAGS5:	SELENL3  	(see note 1) set to 1 it enables MENU line 3 to be selected in the menu, 0 disables selection of the line
//	FLAGS6:	unused
//	FLAGS7:	unused
//
//	(note 1:  the SELENn bits are now ignored, and the selction is disabled only if NOTGT is specified as the line's target (ie #define MENU_m_LNn_TGT NOTGT))
//
//




//The Menu Constants
#define NUM_MENUS	12

#define NOTGT		0xFF					//DISABLES the menu Line's targeting (ie, the highest value of a menu_index_t)
#define BACKTGT		0xFE					//SIGNALS return to previous menu (ie pop_menu())
#define FUNC		0xFD					//signals that the target of the line is a function whose pointer is specified in that line's FP
#define CUSTOM		0xFC					//signals that the line's target is handled by a custom handler function, (ie you have to write that: cf the CANCEL-CONFIRM MENU)
#define CLOCK		0xFB
//#define ONSELECT	0xFA					//calls the associated function pointer when the line is selected




//---------------------------------------------------------------------------
//MENU DEFINITION SECTION
//---------------------------------------------------------------------------
//This is the way to define a menu
//MENU 0 is the MAIN screen
#define MENU_0_NUM		0				//the menu number
#define MENU_0_LN0_TXT	BU				//line 0 text (the text displayed on line 0, or NULL)
#define MENU_0_LN0_TGT	NOTGT			//line 0 target	(the menu number of the targeted menu, or any one of the defined menu constants)
#define MENU_0_LN0_FP	NULL			//line 0 function pointer if FUNC is specified in TGT, or NULL 
#define MENU_0_LN1_TXT	PRODUCT			//line 1 text
#define MENU_0_LN1_TGT	NOTGT			//line 1 target
#define MENU_0_LN1_FP	NULL			//line 1 function pointer
#define MENU_0_LN2_TXT	FWV				//line 2 text
#define MENU_0_LN2_TGT	CLOCK			//line 2 target
#define MENU_0_LN2_FP	NULL			//line 2 function pointer
#define MENU_0_LN3_TXT	"MENU"			//line 3 text
#define MENU_0_LN3_TGT	1				//line 3 target
#define MENU_0_LN3_FP	NULL			//line 3 function pointer
#define MENU_0_FLAGS	0x03			//default selected line and line select enable bits
#define MENU_0_ONLOAD	main_menu_start
#define MENU_0_ONUNLOAD	main_menu_stop


//MENU 1 is the MENU screen
#define MENU_1_NUM		1					
#define MENU_1_LN0_TXT	"Set Animal Number"	
#define MENU_1_LN0_TGT	7
#define MENU_1_LN0_FP	NULL
#define MENU_1_LN1_TXT	"Terminal Mode"
#define MENU_1_LN1_TGT	FUNC
#define MENU_1_LN1_FP	terminal_mode
#define MENU_1_LN2_TXT	"More"		
#define MENU_1_LN2_TGT	6
#define MENU_1_LN2_FP	NULL
#define MENU_1_LN3_TXT	"Back"	
#define MENU_1_LN3_TGT	BACKTGT
#define MENU_1_LN3_FP	NULL
#define MENU_1_FLAGS	0x00			
#define MENU_1_ONLOAD	NULL
#define MENU_1_ONUNLOAD	NULL

//MENU 2 is the LEFT CHANNEL 
#define MENU_2_NUM		2					
#define MENU_2_LN0_TXT	"LEFT CHANNEL"	
#define MENU_2_LN0_TGT	NOTGT
#define MENU_2_LN0_FP	NULL
#define MENU_2_LN1_TXT	""
#define MENU_2_LN1_TGT	NOTGT
#define MENU_2_LN1_FP	NULL
#define MENU_2_LN2_TXT	""		
#define MENU_2_LN2_TGT	NOTGT
#define MENU_2_LN2_FP	NULL
#define MENU_2_LN3_TXT	"START"	
#define MENU_2_LN3_TGT	FUNC
#define MENU_2_LN3_FP	start_left_channel
#define MENU_2_FLAGS	0x03
#define MENU_2_ONLOAD	generate_channel_display
#define MENU_2_ONUNLOAD	destroy_channel_display


//MENU 3 is the RIGHT CHANNEL 
#define MENU_3_NUM		3					
#define MENU_3_LN0_TXT	"RIGHT CHANNEL"	
#define MENU_3_LN0_TGT	NOTGT
#define MENU_3_LN0_FP	NULL
#define MENU_3_LN1_TXT	""
#define MENU_3_LN1_TGT	NOTGT
#define MENU_3_LN1_FP	NULL
#define MENU_3_LN2_TXT	""		
#define MENU_3_LN2_TGT	NOTGT
#define MENU_3_LN2_FP	NULL
#define MENU_3_LN3_TXT	"START"	
#define MENU_3_LN3_TGT	FUNC
#define MENU_3_LN3_FP	start_right_channel
#define MENU_3_FLAGS	0x03
#define MENU_3_ONLOAD	generate_channel_display
#define MENU_3_ONUNLOAD	destroy_channel_display

//MENU 4 Set Time Menu
#define MENU_4_NUM		4					
#define MENU_4_LN0_TXT	"MM/DD/YYYY  HH:MM:SS"	
#define MENU_4_LN0_TGT	CLOCK
#define MENU_4_LN0_FP	NULL
#define MENU_4_LN1_TXT	""
#define MENU_4_LN1_TGT	NOTGT
#define MENU_4_LN1_FP	NULL
#define MENU_4_LN2_TXT	"SET"		
#define MENU_4_LN2_TGT	FUNC
#define MENU_4_LN2_FP	set_date_time
#define MENU_4_LN3_TXT	"BACK"	
#define MENU_4_LN3_TGT	BACKTGT
#define MENU_4_LN3_FP	NULL
#define MENU_4_FLAGS	0x03	
#define MENU_4_ONLOAD	start_menu_clock
#define MENU_4_ONUNLOAD	stop_menu_clock

//MENU 5 VIEW TRIALS MENU
#define MENU_5_NUM		5					
#define MENU_5_LN0_TXT	""	
#define MENU_5_LN0_TGT	NOTGT
#define MENU_5_LN0_FP	NULL
#define MENU_5_LN1_TXT	""
#define MENU_5_LN1_TGT	NOTGT
#define MENU_5_LN1_FP	NULL
#define MENU_5_LN2_TXT	""		
#define MENU_5_LN2_TGT	NOTGT
#define MENU_5_LN2_FP	NULL
#define MENU_5_LN3_TXT	"DONE"	
#define MENU_5_LN3_TGT	BACKTGT
#define MENU_5_LN3_FP	NULL
#define MENU_5_FLAGS	0x03
#define MENU_5_ONLOAD	generate_trials_display
#define MENU_5_ONUNLOAD	destroy_trials_display


//MENU 6 Expansion Menu 
#define MENU_6_NUM		6					
#define MENU_6_LN0_TXT	"View Trials"	
#define MENU_6_LN0_TGT	5
#define MENU_6_LN0_FP	NULL
#define MENU_6_LN1_TXT	"Set Time"
#define MENU_6_LN1_TGT	4
#define MENU_6_LN1_FP	NULL
#define MENU_6_LN2_TXT	"Help"		
#define MENU_6_LN2_TGT	9
#define MENU_6_LN2_FP	NULL
#define MENU_6_LN3_TXT	"BACK"	
#define MENU_6_LN3_TGT	BACKTGT
#define MENU_6_LN3_FP	NULL
#define MENU_6_FLAGS	0x00
#define MENU_6_ONLOAD	NULL
#define MENU_6_ONUNLOAD	NULL

//MENU 7 	Set Aminal Number Menu
#define MENU_7_NUM		7					
#define MENU_7_LN0_TXT	"ANIMAL NUMBER"	
#define MENU_7_LN0_TGT	NOTGT
#define MENU_7_LN0_FP	NULL
#define MENU_7_LN1_TXT	"?"
#define MENU_7_LN1_TGT	NOTGT
#define MENU_7_LN1_FP	NULL
#define MENU_7_LN2_TXT	"SET"		
#define MENU_7_LN2_TGT	FUNC
#define MENU_7_LN2_FP	user_set_animal_number
#define MENU_7_LN3_TXT	"BACK"	
#define MENU_7_LN3_TGT	BACKTGT
#define MENU_7_LN3_FP	NULL
#define MENU_7_FLAGS	0x03
#define MENU_7_ONLOAD	display_animal_number
#define MENU_7_ONUNLOAD	destroy_animal_number


//MENU 8	NOT YET IMPLEMENTED MENU
#define MENU_8_NUM		8					
#define MENU_8_LN0_TXT	"This feature is not"	
#define MENU_8_LN0_TGT	NOTGT
#define MENU_8_LN0_FP	NULL
#define MENU_8_LN1_TXT	"yet implemented."
#define MENU_8_LN1_TGT	NOTGT
#define MENU_8_LN1_FP	NULL
#define MENU_8_LN2_TXT	""		
#define MENU_8_LN2_TGT	NOTGT
#define MENU_8_LN2_FP	NULL
#define MENU_8_LN3_TXT	"BACK"	
#define MENU_8_LN3_TGT	BACKTGT
#define MENU_8_LN3_FP	NULL
#define MENU_8_FLAGS	0x03
#define MENU_8_ONLOAD	NULL
#define MENU_8_ONUNLOAD	NULL


//MENU 9 	HELP MENU
#define MENU_9_NUM		9					
#define MENU_9_LN0_TXT	WEBSITE0	
#define MENU_9_LN0_TGT	NOTGT
#define MENU_9_LN0_FP	NULL
#define MENU_9_LN1_TXT	WEBSITE1
#define MENU_9_LN1_TGT	NOTGT
#define MENU_9_LN1_FP	NULL
#define MENU_9_LN2_TXT	WEBSITE2
#define MENU_9_LN2_TGT	NOTGT
#define MENU_9_LN2_FP	NULL
#define MENU_9_LN3_TXT	"BACK"	
#define MENU_9_LN3_TGT	BACKTGT
#define MENU_9_LN3_FP	NULL
#define MENU_9_FLAGS	0x03
#define MENU_9_ONLOAD	NULL
#define MENU_9_ONUNLOAD	NULL

//MENU 10	CANCEL-CONFIRM MENU 
#define MENU_10_NUM			10					
#define MENU_10_LN0_TXT		"Cancel"	
#define MENU_10_LN0_TGT		CUSTOM
#define MENU_10_LN0_FP		NULL
#define MENU_10_LN1_TXT		"Confirm"
#define MENU_10_LN1_TGT		CUSTOM
#define MENU_10_LN1_FP		NULL
#define MENU_10_LN2_TXT		""		
#define MENU_10_LN2_TGT		NOTGT
#define MENU_10_LN2_FP		NULL
#define MENU_10_LN3_TXT		""	
#define MENU_10_LN3_TGT		NOTGT
#define MENU_10_LN3_FP		NULL
#define MENU_10_FLAGS		0x00
#define MENU_10_ONLOAD		NULL
#define MENU_10_ONUNLOAD	NULL


//MENU 11	TERMINAL MODE MENU 
#define MENU_11_NUM			11					
#define MENU_11_LN0_TXT		"TERMINAL MODE"	
#define MENU_11_LN0_TGT		NOTGT
#define MENU_11_LN0_FP		NULL
#define MENU_11_LN1_TXT		""
#define MENU_11_LN1_TGT		NOTGT
#define MENU_11_LN1_FP		NULL
#define MENU_11_LN2_TXT		""		
#define MENU_11_LN2_TGT		NOTGT
#define MENU_11_LN2_FP		NULL
#define MENU_11_LN3_TXT		"CANCEL"	
#define MENU_11_LN3_TGT		CUSTOM
#define MENU_11_LN3_FP		NULL
#define MENU_11_FLAGS		0x03
#define MENU_11_ONLOAD		NULL
#define MENU_11_ONUNLOAD	NULL


//---------------------------------------------------------------------------
//END OF MENU DEFINITION SECTION
//---------------------------------------------------------------------------






#endif
